use strict;
use Module::Build;

my $build = Module::Build->new(
    create_makefile_pl => 'passthrough',
    license            => 'perl',
    module_name        => 'Catalyst::Plugin::Unicode::Encoding',
    requires           => {
        'Catalyst' => 5.20,
        'Encode'   => 2.10
    },
    create_makefile_pl => 'passthrough',
    test_files   => [
        glob('t/*.t') 
    ]
);

$build->create_build_script;
